Type: Package
Package: gtfsio
Title: Read and Write General Transit Feed Specification (GTFS) Files
Version: 1.2.1
Authors@R: 
    c(person(given = "Daniel",
             family = "Herszenhut",
             role = c("aut"),
             email = "dhersz@gmail.com",
             comment = c(ORCID = "0000-0001-8066-1105")),
      person(given = "Flavio",
             family = "Poletti",
             role = c("aut", "cre"),
             email = "flavio.poletti@hotmail.ch"),
      person(given = "Mark",
             family = "Padgham",
             role = "aut",
             email = "mark.padgham@email.com"),
      person(given = "Rafael H. M.",
             family = "Pereira",
             role = "rev",
             email = "rafa.pereira.br@gmail.com",
             comment = c(ORCID = "0000-0003-2125-7465")),
      person(given = "Tom",
             family = "Buckley",
             role = "rev",
             email = "tom@tbuckl.com"),
      person(given = "Ipea - Institute for Applied Economic Research",
             role = c("cph", "fnd")))
Description: Tools for the development of packages related to General
    Transit Feed Specification (GTFS) files. Establishes a standard for
    representing GTFS feeds using R data types. Provides fast and flexible
    functions to read and write GTFS feeds while sticking to this
    standard. Defines a basic 'gtfs' class which is meant to be extended
    by packages that depend on it. And offers utility functions that
    support checking the structure of GTFS objects.
License: MIT + file LICENSE
URL: https://r-transit.github.io/gtfsio/,
        https://github.com/r-transit/gtfsio
BugReports: https://github.com/r-transit/gtfsio/issues
Imports: data.table, fs, utils, zip, jsonlite
Suggests: knitr, rmarkdown, tinytest
VignetteBuilder: knitr
Encoding: UTF-8
RoxygenNote: 7.3.3
Collate: 'gtfsio_error.R' 'assert_gtfs.R' 'assert_inputs.R' 'checks.R'
        'data.R' 'export_gtfs.R' 'get_gtfs_standards.R'
        'gtfs_methods.R' 'gtfs_subset.R' 'gtfsio.R' 'import_gtfs.R'
        'new_gtfs.R' 'utils.R'
LazyData: true
Depends: R (>= 3.5)
NeedsCompilation: no
Packaged: 2026-05-19 15:24:40 UTC; flaviopoletti
Author: Daniel Herszenhut [aut] (ORCID:
    <https://orcid.org/0000-0001-8066-1105>),
  Flavio Poletti [aut, cre],
  Mark Padgham [aut],
  Rafael H. M. Pereira [rev] (ORCID:
    <https://orcid.org/0000-0003-2125-7465>),
  Tom Buckley [rev],
  Ipea - Institute for Applied Economic Research [cph, fnd]
Maintainer: Flavio Poletti <flavio.poletti@hotmail.ch>
Repository: CRAN
Date/Publication: 2026-05-20 06:20:46 UTC
Built: R 4.5.2; ; 2026-05-20 21:41:22 UTC; unix
