# Generated by roxygen2: do not edit by hand

S3method("[",gtfs)
S3method(print,gtfs)
S3method(summary,gtfs)
export(assert_field_class)
export(assert_field_exists)
export(assert_file_exists)
export(assert_gtfs)
export(check_field_class)
export(check_field_exists)
export(check_file_exists)
export(export_gtfs)
export(get_gtfs_standards)
export(import_gtfs)
export(new_gtfs)
importFrom(data.table,"%chin%")
importFrom(data.table,":=")
importFrom(jsonlite,read_json)
